##############################################################################
# Copyright (c) 2005 Rob Miller (and contributors)
# All Rights Reserved.
#
# This software is subject to the provisions of the GNU General Public
# License, Version 2.0 (GPL).  A copy of the GPL should accompany this
# distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
# AGAINST INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
##############################################################################

"""
Install Samplex, the Sample Extranet

$Id:$
"""

from StringIO import StringIO

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import _createObjectByType
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.Archetypes import listTypes
from Products.CMFMember.Extensions.toolbox import SetupMember

from Products.Samplex.config import PROJECT_NAME, GLOBALS, DEPENDENCIES, \
     TEAMSPACES_FOLDER_DATA, TEAM_ROLES
from Products.Samplex.permissions import DEFAULT_PERMISSIONS_DATA
from Products.Samplex.workflows import DEFAULT_WORKFLOW, WORKFLOW_MAP
from Products.Samplex.content.member import SamplexMember
from Products.Samplex.content.team import SamplexTeam

def installDependencies(portal, out):
    qi = getToolByName(portal, 'portal_quickinstaller')
    for product in DEPENDENCIES:
        if not qi.isProductInstalled(product):
            if qi.isProductInstallable(product):
                qi.installProduct(product)
            else:
                raise "Product %s not installable" % product

def installRoles(portal, out):
    """ Installs custom roles """
    vr = portal.validRoles()

    for role in TEAM_ROLES:
        if not role in vr:
            print >> out, "-> Adding '%s'" % role
            portal.manage_defined_roles(submit='Add Role',
                                        REQUEST={'role': role})

def installWorkflows(portal, out):
    """ Installs workflows """
    wfs = WORKFLOW_MAP
    wf_tool = getToolByName(portal, 'portal_workflow')
    existing_wfs = wf_tool.objectIds()

    for wf in wfs:
        if not wf in existing_wfs:
            print >> out, '-> Adding %s' % wf
            wf_tool.manage_addWorkflow('%s (%s)' % (wf, wfs[wf][0]), wf)
        print >> out, '-> Assigning types to %s' % wf
        wf_tool.setChainForPortalTypes(wfs[wf][1], wf)

    wf_tool.setDefaultChain(DEFAULT_WORKFLOW)
    wf_tool.updateRoleMappings()


def installTeamSpacesFolder(portal, out):
    id = TEAMSPACES_FOLDER_DATA['id']
    title = TEAMSPACES_FOLDER_DATA['title']
    if not id in portal.contentIds(filter={'portal_type':
                                           'Large Plone Folder'}):
        print >> out, 'Adding teamspaces folder'
        _createObjectByType('Large Plone Folder', portal, id,
                            title=title)

def replaceListItem(repl_list, orig_item, new_item):
    """ replace item in a list with a specified new item; do nothing
        if the original item is not in the list """
    try:
        index = repl_list.index(orig_item)
        repl_list = repl_list[:index] + [new_item] + repl_list[index+1:]
        return repl_list
    except ValueError:
        pass

def installPortlets(portal, out):
    """ install additional portlets """
    print >> out, 'Adding "my spaces" portlet'
    portlet_form = 'here/%s/macros/portlet'
    nav_portlet = portlet_form % 'portlet_navigation'
    spaces_portlet = portlet_form % 'portlet_my_spaces'
    left_slots = list(portal.getProperty('left_slots'))
    if not spaces_portlet in left_slots:
        try:
            nav_index = left_slots.index(nav_portlet)
        except ValueError:
            nav_index = -1
        left_slots = left_slots[:nav_index+1] + \
                     [spaces_portlet] + \
                     left_slots[nav_index+1:]
    portal.manage_changeProperties(left_slots=tuple(left_slots))

    news_portlet = portlet_form % 'portlet_news'
    calendar_portlet = portlet_form % 'portlet_calendar'
    tm_news_portlet = portlet_form % 'portlet_team_news'
    tm_calendar_portlet = portlet_form % 'portlet_team_calendar'
    right_slots = list(portal.getProperty('right_slots'))
    right_slots = replaceListItem(right_slots, news_portlet,
                                  tm_news_portlet)
    right_slots = replaceListItem(right_slots, calendar_portlet,
                                  tm_calendar_portlet)
    portal.manage_changeProperties(right_slots=tuple(right_slots))

def securityTweaks(portal, out):
    """ tweak site-wide security settings """
    for roles, perms in DEFAULT_PERMISSIONS_DATA:
        for perm in perms:
            print >> out, '-> Specifying "%s" permission access' % perm
            portal.manage_permission(perm, roles=roles)

def setDefaultTeamPolicy(self, out):
    """ register team roles and team types """
    print >> out, 'Specifying team and team role policy'
    tm_tool = getToolByName(self, 'portal_teams')
    print >> out, '-> Setting allowed and default team roles'
    tm_tool.setDefaultAllowedRoles(TEAM_ROLES)
    tm_tool.setDefaultRoles(TEAM_ROLES[:1])
    allowed_team_types = tm_tool.getAllowedTeamTypes()
    if not SamplexTeam.portal_type in allowed_team_types:
        print >> out, '-> Adding %s to allowed team types' \
              % SamplexTeam.portal_type
        allowed_team_types += (SamplexTeam.portal_type,)
        tm_tool.setAllowedTeamTypes(allowed_team_types)


def install(self):
    out = StringIO()
    portal = getToolByName(self, 'portal_url').getPortalObject()
    
    # Install dependency products
    installDependencies(portal, out)

    # Install types
    installTypes(self, out, listTypes(PROJECT_NAME), PROJECT_NAME)

    # Install skins
    install_subskin(self, out, GLOBALS, 'skins')

    # Install add'l roles
    installRoles(portal, out)

    # Install custom workflows
    installWorkflows(portal, out)

    # Install teamspaces folder
    installTeamSpacesFolder(portal, out)

    # Install portlets
    installPortlets(portal, out)

    # Implement site-wide security adjustments
    securityTweaks(portal, out)

    # Specify default team and team role policy w/ team tool
    setDefaultTeamPolicy(portal, out)
    
    # do the rest of the member type dirty work
    # (catalogs, workflow, registration)
    print >> out, SetupMember(self, member_type=SamplexMember.portal_type).finish()

    print >> out, "Successfully installed %s." % PROJECT_NAME
    return out.getvalue()
