##############################################################################
# Copyright (c) 2005 Rob Miller (and contributors)
# All Rights Reserved.
#
# This software is subject to the provisions of the GNU General Public License,
# Version 2.0 (GPL).  A copy of the GPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
##############################################################################

"""
Custom member class

$Id:$
"""

from AccessControl import ClassSecurityInfo, ModuleSecurityInfo

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import *

from Products.CMFMember.api import Member as BaseMember
from Products.CMFMember.api import id_schema, plone_schema, plone_2_1_schema, \
     security_schema, login_info_schema
from Products.CMFMember.permission import VIEW_PUBLIC_PERMISSION, \
     EDIT_PROPERTIES_PERMISSION

contact_schema = Schema((
    StringField('firstname',
                default='',
                mode='rw',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                required=1,
                searchable=1,
                index='ZCTextIndex|schema',
                widget=StringWidget(label='First name',
                                    description='Enter your first name here.',
                                    label_msgid='label_firstname',
                                    description_msgid='desc_firstname',),
                regfield=1,  ### this field is part of the registration form
                ),

    StringField('lastname',
                default='',
                mode='rw',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                required=1,
                searchable=1,
                index='ZCTextIndex|schema',
                widget=StringWidget(label='Last name',
                                    description='Enter your last name here.',
                                    label_msgid='label_lastname',
                                    description_msgid='desc_lastname',),
                regfield=1,  ### this field is part of the registration form
                ),

    #this field computes the 'fullname'
    #fullname = 'firstname'+'lastname'                
    ComputedField('fullname',
                  read_permission=VIEW_PUBLIC_PERMISSION,
                  expression='context.computeFullname()',
                  searchable=1,
                  index='ZCTextIndex|schema',
                  widget=StringWidget(label='',
                                      description='',
                                      label_msgid='label_fullname',
                                      description_msgid='desc_fullname',
                                      visible={'view': 'visible',
                                               'edit': 'invisible',
                                               'roster_list': 'visible'},
                                      ),
                  ),

    StringField('email',
                required=1,
                mode='rw',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                validators=('isEmail',),
                index='FieldIndex|schema',
                searchable=1,
                widget=StringWidget(label='E-mail',
                                    description='Enter your e-mail address here.',
                                    label_msgid='label_email',
                                    description_msgid='desc_email',
                                    visible={'view': 'visible',
                                             'edit': 'visible',
                                             'roster_list': 'visible'},
                                    ),
                regfield=1,  ### this field is part of the registration form
                ),

    # code modified from Joel Burton's Foundation Member class
    StringField('address',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                index='ZCTextIndex|schema',
                searchable=1,
                widget=TextAreaWidget(label='Address',
                                      rows=3,
                                      cols=40,
                                      label_msgid='label_address',
                                      description_msgid='desc_address',),
                schemata='Address',
                ),

    StringField('city',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                index='FieldIndex|schema',
                searchable=1,
                widget=StringWidget(label='City',
                                    size=25,
                                    label_msgid='label_city',
                                    description_msgid='desc_city',
                                    visible={'view':'visible',
                                             'edit':'visible',
                                             'roster_list':'visible',},
                                    ),
                schemata='Address',
                ),

    StringField('state',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                index='FieldIndex|schema',
                searchable=1,
                widget=StringWidget(label='State/Region',
                                    size=25,
                                    label_msgid='label_state',
                                    visible={'view':'visible',
                                             'edit':'visible',
                                             'roster_list':'visible',},
                                    description_msgid='desc_state',),
                schemata='Address',
                ),

    StringField('postalCode',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                index='FieldIndex|schema',
                searchable=1,
                widget=StringWidget(label='Postal Code/Zip Code',
                                    size=25,
                                    label_msgid='label_postalCode',
                                    description_msgid='desc_postalCode',),
                schemata='Address',
                ),

    StringField('country',
                read_permission=VIEW_PUBLIC_PERMISSION,
                write_permission=EDIT_PROPERTIES_PERMISSION,
                index='FieldIndex|schema',
                searchable=1,
                widget=StringWidget(label='Country',
                                    size=40,
                                    label_msgid='label_country',
                                    visible={'view':'visible',
                                             'edit':'visible',
                                             'roster_list':'visible',},
                                    description_msgid='desc_country'),
                schemata='Address',
                ),
    ))


team_membership_schema = Schema((

    ComputedField('active_teams',
                  read_permission=VIEW_PUBLIC_PERMISSION,
                  expression='context.getActiveTeams()',
                  widget=ComputedWidget(
                        label="Teams",
                        label_msgid="label_active_teams",
                        description="Teams on which this member is active.",
                        description_msgid="desc_active_teams",
                        visible={'edit': 'visible',
                                 'view': 'visible'},
                        ),
                  ),
    ))


content_schema = id_schema + contact_schema + team_membership_schema + \
                 plone_schema + plone_2_1_schema + security_schema + \
                 login_info_schema


class SamplexMember(BaseMember):
    security = ClassSecurityInfo()
    schema = content_schema + ExtensibleMetadata.schema
    portal_type = meta_type = 'SamplexMember'
    archetype_name='Samplex Member'

    global_allow = 0

    def computeFullname(self):
        """ compute the fullname from the firstname and lastname values """
        firstname = self.getField('firstname').getAccessor(self)()
        lastname = self.getField('lastname').getAccessor(self)()
        if firstname and lastname:
            space = ' '
        else:
            space = ''
        return "%s%s%s" % (firstname, space, lastname)

    def getActiveTeams(self):
        """ returns a list of team objects for which the member is currently
            active """
        tm_tool = getToolByName(self, 'portal_teams')
        return [tm.getId() for tm in \
                tm_tool.getTeamsByMemberId(self.getId(), active=1)]


registerType(SamplexMember)
