##############################################################################
# Copyright (c) 2005 Rob Miller (and contributors)
# All Rights Reserved.
#
# This software is subject to the provisions of the GNU General Public
# License, Version 2.0 (GPL).  A copy of the GPL should accompany this
# distribution.  THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL
# EXPRESS OR IMPLIED WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
# AGAINST INFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
##############################################################################

"""
Test the basic installation of the Samplex components

$Id:$
"""

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CMFCore.utils import getToolByName
from Products.Archetypes import listTypes

from Products.Samplex.Extensions.Install import install
from Products.Samplex.config import PROJECT_NAME, DEPENDENCIES, TEAM_ROLES, \
     TEAMSPACES_FOLDER_DATA
from Products.Samplex.permissions import DEFAULT_PERMISSIONS_DATA
from Products.Samplex.workflows import DEFAULT_WORKFLOW, WORKFLOW_MAP
from Products.Samplex.content.member import SamplexMember
from Products.Samplex.content.team import SamplexTeam
from base import SamplexTestCase

class TestInstall(SamplexTestCase):
    """ basic test for installation, qi """
    
    def afterSetUp(self):
        """ do nothing to enable clean test of install """
        pass

    def testQI(self):
        try:
            self.installProducts([PROJECT_NAME])
        except:
            self.fail('QI install failed')
            
    def testInstallMethod(self):
        try:
            install(self.portal)
        except:
            self.fail('Install from method failed')

class TestInstallEffects(SamplexTestCase):
    """ make sure the installation has the desired effects on the site """
    
    def testDependenciesInstalled(self):
        qi = getToolByName(self.portal, 'portal_quickinstaller')
        for product in DEPENDENCIES:
            self.failUnless(qi.isProductInstalled(product),
                            'Dependency product "%s" is not installed' % product)

    def testTypesInstalled(self):
        ttool = getToolByName(self.portal, 'portal_types')
        product_types = [t['portal_type'] for t in listTypes(PROJECT_NAME)]
        installed_types = ttool.listContentTypes()
        for product_type in product_types:
            self.failUnless(product_type in installed_types,
                            '%s type not installed' % product_type)

    def testRolesInstalled(self):
        for role in TEAM_ROLES:
            vr = self.portal.validRoles()
            self.failUnless(role in vr,
                            "%s role not installed" % role)

    def testWorkflowsInstalled(self):
        wf_tool= getToolByName(self.portal, 'portal_workflow')
        for wf, wf_data in WORKFLOW_MAP.items():
            for type in wf_data[1]:
                # we're assuming only one workflow per type
                chain = wf_tool.getChainForPortalType(type)
                self.failUnless(chain[0]==wf,
                                'Workflow for type %s should be %s' % (type, wf))

        def_wf = wf_tool._default_chain[0]
        self.failUnless(def_wf == DEFAULT_WORKFLOW,
                        'Default workflow should be %s' % def_wf)

    def testPortletsInstalled(self):
        portlet_form = 'here/%s/macros/portlet'
        nav_portlet = portlet_form % 'portlet_navigation'
        spaces_portlet = portlet_form % 'portlet_my_spaces'
        left_slots = list(self.portal.getProperty('left_slots'))
        try:
            nav_index = left_slots.index(nav_portlet)
        except ValueError:
            nav_index = -1
        spaces_index = left_slots.index(spaces_portlet)
        self.failUnless(spaces_index - nav_index == 1,
                        '"my_spaces" portlet not added to left_slots')

        tm_news_portlet = portlet_form % 'portlet_team_news'
        tm_calendar_portlet = portlet_form % 'portlet_team_calendar'
        right_slots = list(self.portal.getProperty('right_slots'))
        for portlet in [tm_news_portlet, tm_calendar_portlet]:
            self.failUnless(portlet in right_slots,
                            '"%s" not in right_slots' % portlet)

    def testTeamSpacesFolderInstalled(self):
        id = TEAMSPACES_FOLDER_DATA['id']
        self.failUnless(id in \
                        self.portal.contentIds(filter={'portal_type':
                                                       'Large Plone Folder'}),
                        '"%s" folder not installed' % id)

    def testTeamPermissionsAreSet(self):
        for roles, perms in DEFAULT_PERMISSIONS_DATA:
            roles_copy = roles[:]
            roles_copy.sort()
            for perm in perms:
                assigned_roles = [r['name']
                                  for r in self.portal.rolesOfPermission(perm)
                                  if r['selected']]
                assigned_roles.sort()
                self.failUnless(assigned_roles == roles_copy,
                                '"%s" permission not granted appropriately' % perm)

    def testTeamPolicyIsSet(self):
        tm_tool = getToolByName(self.portal, 'portal_teams')
        self.failUnless(tm_tool.getDefaultAllowedRoles() == TEAM_ROLES,
                        'Default allowed team roles not set correctly')
        self.failUnless(tm_tool.getDefaultRoles() == TEAM_ROLES[:1],
                        'Default team role not set correctly')
        self.failIf(SamplexTeam.portal_type not in tm_tool.getAllowedTeamTypes(),
                    '"%s" team type not allowed by team tool' \
                    % SamplexTeam.portal_type)

    def testDefaultMember(self):
        mdc = getToolByName(self.portal, 'portal_memberdata')
        self.failUnless(mdc.getTypeName() == SamplexMember.portal_type)
        

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestInstall))
    suite.addTest(makeSuite(TestInstallEffects))    
    return suite

if __name__ == '__main__':
    framework()
